@@ -18,6 +18,7 @@
  */
 import React from 'react';
 import PropTypes from 'prop-types';
+import { styled } from '@superset-ui/core';
 
 import DashboardComponent from '../../containers/DashboardComponent';
 import DragDroppable from '../dnd/DragDroppable';
@@ -62,6 +63,17 @@ const defaultProps = {
   onResizeStop() {},
 };
 
+const TabTitleContainer = styled.div`
+  ${({ isHighlighted, theme: { gridUnit, colors } }) => `
+    padding: ${gridUnit}px ${gridUnit * 2}px;
+    margin: ${-gridUnit}px ${gridUnit * -2}px;
+    transition: box-shadow 0.2s ease-in-out;
+    ${
+      isHighlighted && `box-shadow: 0 0 ${gridUnit}px ${colors.primary.light1};`
+    }
+  `}
+`;
+
 export default class Tab extends React.PureComponent {
   constructor(props) {
     super(props);
@@ -192,6 +204,7 @@ export default class Tab extends React.PureComponent {
       editMode,
       filters,
       isFocused,
+      isHighlighted,
     } = this.props;
 
     return (
@@ -205,12 +218,19 @@ export default class Tab extends React.PureComponent {
         editMode={editMode}
       >
         {({ dropIndicatorProps, dragSourceRef }) => (
-          <div className="dragdroppable-tab" ref={dragSourceRef}>
+          <TabTitleContainer
+            isHighlighted={isHighlighted}
+            className="dragdroppable-tab"
+            ref={dragSourceRef}
+          >
             <EditableTitle
               title={component.meta.text}
+              defaultTitle={component.meta.defaultText}
+              placeholder={component.meta.placeholder}
               canEdit={editMode && isFocused}
               onSaveTitle={this.handleChangeText}
               showTooltip={false}
+              editing={editMode && isFocused}
             />
             {!editMode && (
               <AnchorLink
@@ -222,7 +242,7 @@ export default class Tab extends React.PureComponent {
             )}
 
             {dropIndicatorProps && <div {...dropIndicatorProps} />}
-          </div>
+          </TabTitleContainer>
         )}
       </DragDroppable>
     );
