@@ -32,13 +32,21 @@ import {
   TOGGLE_PUBLISHED,
   UPDATE_CSS,
   SET_REFRESH_FREQUENCY,
+  ON_REFRESH,
+  ON_REFRESH_SUCCESS,
   SET_DIRECT_PATH,
   SET_FOCUSED_FILTER_FIELD,
   UNSET_FOCUSED_FILTER_FIELD,
+  SET_ACTIVE_TABS,
+  SET_FULL_SIZE_CHART_ID,
 } from '../actions/dashboardState';
+import { HYDRATE_DASHBOARD } from '../actions/hydrate';
 
 export default function dashboardStateReducer(state = {}, action) {
   const actionHandlers = {
+    [HYDRATE_DASHBOARD]() {
+      return { ...state, ...action.data.dashboardState };
+    },
     [UPDATE_CSS]() {
       return { ...state, css: action.css };
     },
@@ -122,6 +130,18 @@ export default function dashboardStateReducer(state = {}, action) {
         hasUnsavedChanges: action.isPersistent,
       };
     },
+    [ON_REFRESH]() {
+      return {
+        ...state,
+        isRefreshing: true,
+      };
+    },
+    [ON_REFRESH_SUCCESS]() {
+      return {
+        ...state,
+        isRefreshing: false,
+      };
+    },
     [SET_DIRECT_PATH]() {
       return {
         ...state,
@@ -129,6 +149,12 @@ export default function dashboardStateReducer(state = {}, action) {
         directPathLastUpdated: Date.now(),
       };
     },
+    [SET_ACTIVE_TABS]() {
+      return {
+        ...state,
+        activeTabs: action.tabIds,
+      };
+    },
     [SET_FOCUSED_FILTER_FIELD]() {
       return {
         ...state,
@@ -154,6 +180,12 @@ export default function dashboardStateReducer(state = {}, action) {
         focusedFilterField: null,
       };
     },
+    [SET_FULL_SIZE_CHART_ID]() {
+      return {
+        ...state,
+        fullSizeChartId: action.chartId,
+      };
+    },
   };
 
   if (action.type in actionHandlers) {
